//=============================================================================
//
//  CLASS Metaball - IMPLEMENTATION
//
//=============================================================================
#define RAYTRACER_SPHERE_CPP

//== INCLUDES =================================================================
#include "Metaball.h"

//== IMPLEMENTATION ==========================================================
//-----------------------------------------------------------------------------
bool Metaball::intersect(
    const Ray       &clRay, 
    double        t0,     
    double        t1,     
    TracingContext  &tContext )  const
{

    /*		
		Strahl-Metaball Schnitt
		Die relevanten (bei Schnitt zu setzenden) Felder im TracingContext sind:

		t          - Strahlparameter des Schnittpunkts
		v3Normal   - Normale im Schnittpunkt
		v3HitPoint - Koordinate des Schnittpunktes
		pclShader  - Verweis auf das Material

	*/
	return false;
}

VectorType2 Metaball::cartesian2Polar( VectorType3 &v )  const
{
	/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
	double    z = ( v[2] > 1 ) ? 1 : ( v[2] < -1 ) ? -1 : v[2];
	double    theta = ( double ) acos( z );
	double    phi = ( double ) atan2( v[1], v[0] );
	/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

	if( phi < 0 )
		phi = phi + 2 * ( double ) M_PI;
	return VectorType2( theta, phi );
}

void Metaball::renderGL() const
{
	AABBox bbox(getAABB());
	double radius = bbox.getSize().length()/50;

	for ( int i = 0; i < m_balls.size(); i++)
	{
		//Generate sphere
		int d = 20;
		glPushMatrix();
		glTranslatef( m_balls[i].v3Center[0], m_balls[i].v3Center[1], m_balls[i].v3Center[2] );

		GLfloat step = M_PI / d;
		GLfloat sina, sinb, sinbp, cosbp, cosa, cosb, x, y, z;
		int     count = 0;
		if( m_selected )
		{
			GLfloat mat_ambient[] = { 1, 1, 1 };
			glMaterialfv( GL_FRONT, GL_AMBIENT, mat_ambient );
		}
		else
			m_shader->shadeGL();
		for( GLfloat beta = 0; beta < 2 * M_PI - 0.5 * step; beta += step )
		{
			glBegin( GL_TRIANGLE_STRIP );
			sinb = sin( beta );
			cosb = cos( beta );
			sinbp = sin( beta + step );
			cosbp = cos( beta + step );
			count++;
			glVertex3f( 0, 0, radius );
			glNormal3f( 0, 0, radius );
			for( GLfloat alpha = step; alpha < M_PI - 0.5 * step; alpha += step )
			{
				sina = sin( alpha );
				cosa = cos( alpha );
				x = radius * sina * cosb;
				y = radius * sina * sinb;
				z = radius * cosa;
				glVertex3f( x, y, z );
				glNormal3f( x, y, z );
				count++;
				x = radius * sina * cosbp;
				y = radius * sina * sinbp;
				z = radius * cosa;
				glVertex3f( x, y, z );
				glNormal3f( x, y, z );
				count++;
			}

			count++;
			glVertex3f( 0, 0, -radius );
			glNormal3f( 0, 0, -radius );
			glEnd();
		}

		glPopMatrix();
	}
}

//translate object
void Metaball::translate(
    float   x,  
    float   y,  
    float   z ) 
{
	VectorType3 center = getAABB().getCenter();
	VectorType3 dt(x,y,z);
	dt = dt - center;

	for( int i = 0; i < m_balls.size(); i++ )
	{
		m_balls[i].v3Center += dt;
	}
}

//rotate object
void Metaball::rotate(
    float   /*rotx*/,   
    float   /*roty*/,   
    float   /*rotz*/ )  
{
	VectorType3 center = getAABB().getCenter();

	// rotate around center
	// update m_balls
}

AABBox Metaball::getAABB() const
{
	VectorType3 minv(DBL_MAX,DBL_MAX,DBL_MAX), maxv(-DBL_MAX,-DBL_MAX,-DBL_MAX);
	for( int i = 0; i < m_balls.size(); i++ )
	{
		if ( m_balls[i].v3Center[0] < minv[0] )
			minv[0] = m_balls[i].v3Center[0];

		if ( m_balls[i].v3Center[1] < minv[1] )
			minv[1] = m_balls[i].v3Center[1];

		if ( m_balls[i].v3Center[2] < minv[2] )
			minv[2] = m_balls[i].v3Center[2];

		if ( m_balls[i].v3Center[0] > maxv[0] )
			maxv[0] = m_balls[i].v3Center[0];

		if ( m_balls[i].v3Center[1] > maxv[1] )
			maxv[1] = m_balls[i].v3Center[1];

		if ( m_balls[i].v3Center[2] > maxv[2] )
			maxv[2] = m_balls[i].v3Center[2];
		
	}

	return AABBox( minv, maxv );
}
