//=============================================================================
//
//  CLASS Metaball
//
//=============================================================================
#ifndef RAYTRACER_METABALL_H
#define RAYTRACER_METABALL_H

//== INCLUDES =================================================================
#include "Surface.h"


//== CLASS DEFINITION =========================================================

/*!
	A sphere. The typical first raytracing-surface!
*/
class Metaball : public Surface
{
    //-------------------------------------------------------------- private types
public:
	// p(r) = exp(-bandwidth r^2)
	struct Ball
	{
		Ball() : weight(1), v3Center(0,0,0), bandwidth(1) {}
		Ball ( double w, const VectorType3 c, double _bandwidth ) : weight(w), v3Center(c), bandwidth(_bandwidth) {}
		double weight;
		VectorType3 v3Center;
		double bandwidth;
	};
	// \sum_i=0..n-1 (weight * p_i(r)
	typedef std::vector<Ball> BallVec;

    //-------------------------------------------------------------- private data
private:
	double m_isovalue;
    BallVec m_balls;

    //-------------------------------------------------------------- private methods
private:
	VectorType2 cartesian2Polar( VectorType3 &v ) const;


public:

    //-------------------------------------------------------------- constructors

    //! standard constructor
	Metaball() { }

	Metaball(double isovalue, int numBalls) : m_isovalue(isovalue) { m_balls.reserve(numBalls); }

    
	void addBall( const Ball &ball ) { m_balls.push_back(ball); }
 	void addBall( float weight, VectorType3 center, float b ) { m_balls.push_back( Ball(weight,center,b) ); }
   
	const Ball& getBall(int i) const { return m_balls[i]; }

    AABBox getAABB() const;

    //render object in OpenGL
    void    renderGL() const;

    //translate object
    void    translate( float x, float y, float z );

    //rotate object
    void    rotate( float rotx, float roty, float rotz );

    //get Translation
    void getTranslation(float   *x, float   *y, float   *z )    
    {
		VectorType3 center = getAABB().getCenter();

        *x = center[0];
        *y = center[1];
        *z = center[2];
    };

    //get Rotation
    void getRotation(int *rotx, int *roty, int *rotz ) 
    {
        *rotx = 0;
        *roty = 0;
        *rotz = 0;
    };

    //--------------------------------------------------------------- intersections
    bool    intersect( const Ray       &clRay,
                       const double  t0,
                       const double  t1,
                       TracingContext  &tContext ) const;
};

//=============================================================================
#endif // RAYTRACER_METABALL_H

