//=============================================================================
//
//  CLASS PhongShader
//
//=============================================================================
#ifndef RAYTRACER_PHONGSHADER_H
#define RAYTRACER_PHONGSHADER_H

//== INCLUDES =================================================================
#include "Shader.h"
#include <QGLWidget>

//== CLASS DEFINITION =========================================================

/*!
	Simpe Phong shader

	\author Gero Mueller
*/
class PhongShader : public Shader
{

    //-------------------------------------------------------------- protected data
protected:

    // Members
    ColorType   m_colDiffuse;
    double    m_rShininess;

    //-------------------------------------------------------------- private methods
private:

public:

    //-------------------------------------------------------------- constructors

    //! standard constructor
    PhongShader()
    {
    };

    //! initializing constructor
    PhongShader(
        const ColorType &colAmbient,        
        const ColorType &colSpecular,       
        const ColorType &colRefractive,     
        const double  fRefractiveIndex,   
        const ColorType &colDiffuse,        
        const double  fShininess )  :
    Shader( colAmbient, colSpecular, colRefractive, fRefractiveIndex ),
    m_colDiffuse( colDiffuse ),
    m_rShininess( fShininess )
    {
    };

    //! destructor	
    ~PhongShader()
    {
    };

    //----------------------------------------------------------- element access
public:
    
    void setDiffuseColor(
        const ColorType &colDiffuse )   
    {
        m_colDiffuse = colDiffuse;
    };

    
    const ColorType &getDiffuseColor() const
    {
        return m_colDiffuse;
    };

    
    void setShininess(
        const double  rShininess )    
    {
        m_rShininess = rShininess;
    };

    
    const double getShininess() const
    {
        return m_rShininess;
    };

    //----------------------------------------------------------- functionality
public:
    ColorType   shade( const TracingContext &tContext ) const;
    void        shadeGL() const;
};

//=============================================================================
#endif // RAYTRACER_PHONGSHADER_H
