//=============================================================================
//
//  CLASS SurfaceList - IMPLEMENTATION
//
//=============================================================================
#define RAYTRACER_SURFACELIST_CPP

//== INCLUDES =================================================================
#include "SurfaceList.h"

//== IMPLEMENTATION ==========================================================
void SurfaceList::removeAllSurfaces()
{
    if( m_bOwnsSurfaces )
    {
        SurfaceContainerType::iterator  it = m_surfaceContainer.begin();
        SurfaceContainerType::iterator  it_end = m_surfaceContainer.end();
        for( ; it != it_end; ++it )
        {
            delete( *it );
        }
    }
   
    TriangleSoupContainerType::iterator  ito = m_triangleSoupContainer.begin();
    TriangleSoupContainerType::iterator  ito_end = m_triangleSoupContainer.end();
    for( ; ito != ito_end; ++ito )
    {
        delete( *ito );
    }

    m_triangleSoupContainer.clear();
    m_surfaceContainer.clear();
}

//-----------------------------------------------------------------------------
bool SurfaceList::intersect(
    const Ray       &clRay, 
    const double  t0,     
    const double  t1,     
    TracingContext  &tContext )  const
{
    if( !getAABB().intersect( clRay ) )
        return false;

    bool    b_ret = false;
    tContext.m_t = t1;

    SurfaceContainerType::const_iterator    it = m_surfaceContainer.begin();
    SurfaceContainerType::const_iterator    it_end = m_surfaceContainer.end();
    for( ; it != it_end; ++it )
    {

        //The last ray-intersection parameter restricts the next intersection
        if( (*it)->intersect( clRay, t0, tContext.m_t, tContext ) )
        {
            b_ret = true;
        }
    }

    return b_ret;
}


void SurfaceList::renderGL() const
{
    SurfaceContainerType::const_iterator    it = m_surfaceContainer.begin();
    SurfaceContainerType::const_iterator    it_end = m_surfaceContainer.end();
    for( ; it != it_end; ++it )
    {

        //The last ray-intersection parameter restricts the next intersection
        ( *it )->renderGL();
    }
    
    
    TriangleSoupContainerType::const_iterator  ito = m_triangleSoupContainer.begin();
    TriangleSoupContainerType::const_iterator  ito_end = m_triangleSoupContainer.end();
    for( ; ito != ito_end; ++ito )
    {
        ( *ito )->setDrawnThisCycle( false );
    }
}

//translate object
void SurfaceList::translate(
    float   x,  
    float   y,  
    float   z ) 
{
    SurfaceContainerType::const_iterator    it = m_surfaceContainer.begin();
    SurfaceContainerType::const_iterator    it_end = m_surfaceContainer.end();
    for( ; it != it_end; ++it )
    {
        ( *it )->translate( x, y, z );
    }
}

//rotate object
void SurfaceList::rotate(
    float   rotx,   
    float   roty,   
    float   rotz )  
{
    SurfaceContainerType::const_iterator    it = m_surfaceContainer.begin();
    SurfaceContainerType::const_iterator    it_end = m_surfaceContainer.end();
    for( ; it != it_end; ++it )
    {
        ( *it )->rotate( rotx, roty, rotz );
    }
}

//Choose selected Object
Surface *SurfaceList::selectNext()
{
    SurfaceContainerType::const_iterator    it = m_surfaceContainer.begin();
    SurfaceContainerType::const_iterator    it_end = m_surfaceContainer.end();
    for( ; it != it_end; it++ )
    {
        if( (*it)->getSelected() )
        {
            Surface *firstselected = (*it);
            while( it != it_end && (*it)->getSelected() )
                it++; 
            firstselected->setSelected( false );
            
            if( it != it_end )
            {
                Surface *ret = ( *it );
                ret->setSelected( true );
                return ret;
            }
            break;
        }
    }

    it = m_surfaceContainer.begin();
    ( *it )->setSelected( true );
    return ( *it );
}



void SurfaceList::addSurface( TriangleSoup *pclTriangleSoup )   
{
    m_triangleSoupContainer.push_back( pclTriangleSoup );
    pclTriangleSoup->addTrianglesToScene( this );
};