//=============================================================================
//
//  CLASS Triangle - IMPLEMENTATION
//
//=============================================================================

#define RAYTRACER_TRIANGLE_CPP


//== INCLUDES =================================================================
#include "TriangleSoup.h"
#include "Triangle.h"
#include "Tools.h"

   
//== IMPLEMENTATION ========================================================== 

Triangle::Triangle( TriangleSoup* pclMesh,
		       int   i3VertexIndex[3],
			   int   i3NormalIndex[3]) 
{
	m_pclMesh = pclMesh;
	if ( !m_pclMesh ) std::cerr << "invalid mesh-reference";

	memcpy( m_i3VertexIndex, i3VertexIndex, sizeof(m_i3VertexIndex) );
	memcpy( m_i3NormalIndex, i3NormalIndex, sizeof(m_i3NormalIndex) );
}
	
//-----------------------------------------------------------------------------
	
AABBox 
Triangle::getAABB() const
{
	AABBox cl_aabbox;
	
	cl_aabbox.extendBy( m_pclMesh->getVertexList()[m_i3VertexIndex[0]]);
	cl_aabbox.extendBy( m_pclMesh->getVertexList()[m_i3VertexIndex[1]]);
	cl_aabbox.extendBy( m_pclMesh->getVertexList()[m_i3VertexIndex[2]]);
	return cl_aabbox;
}

//-----------------------------------------------------------------------------
//Schnittfunktion zwischen Strahl und Dreieck
bool 
Triangle::intersect(const Ray& clRay, double t0, double t1, TracingContext& tContext ) const
{	
	//Die 3 Punkte des Dreiecks
	VectorType3 v3_point0 = m_pclMesh->getVertexList()[m_i3VertexIndex[0]]; 
	VectorType3 v3_point1 = m_pclMesh->getVertexList()[m_i3VertexIndex[1]];
	VectorType3 v3_point2 = m_pclMesh->getVertexList()[m_i3VertexIndex[2]];

	VectorType3 u = v3_point1-v3_point0;
	VectorType3 v = v3_point2-v3_point0;
	
	//Der Normalenvektor der durch das Dreieck aufgespannten Ebene
	VectorType3 n = u % v;
	n.normalize();
	
    /*
		Implementieren Sie:
		
		Strahl-Dreieck-Schnittest.
	*/
	return false;
	
	//baryzentrische Koordinaten
	double beta, gamma;
	double t;
	VectorType3 v3_intersection;
	
	
	//Setzen der Parameter fr 
	tContext.m_t = t;
	tContext.m_v3HitPoint = v3_intersection;
	//Nicht in allen OBJ-Dateien werden die Vertex-Normalen auch gesetzt
	//Ist dies nicht der Fall, verwenden wir einfach die Ebenen-Normale
	if ( m_i3NormalIndex[0] != -1 &&
	     m_i3NormalIndex[0] != -1 &&
	     m_i3NormalIndex[0] != -1 )
	    tContext.m_v3Normal = (1-beta-gamma)*m_pclMesh->getNormalList()[m_i3NormalIndex[0]]+
		                              beta*m_pclMesh->getNormalList()[m_i3NormalIndex[1]]+
			    		             gamma*m_pclMesh->getNormalList()[m_i3NormalIndex[2]];
	else
	    tContext.m_v3Normal = n;
	tContext.m_v3Normal.normalize();

	tContext.m_pclShader = m_pclMesh->getShader();
	return true;				
}

//Render in GL Mode
void Triangle::renderGL() const
{
    if ( !m_pclMesh->drawnThisCycle() )
        m_pclMesh->renderGL();
}



int Triangle::getNormalIndex( unsigned int i ) const
{
    if ( i < 3 )
        return m_i3NormalIndex[ i ];
    return -1;
}

int Triangle::getVertexIndex( unsigned int i ) const
{
    if ( i < 3 )
        return m_i3VertexIndex[ i ];
    return -1;
}


void Triangle::setSelected( bool sel )   
{
    m_pclMesh->setSelected( sel );
};


bool Triangle::getSelected() const
{
    return m_pclMesh->getSelected();
};

void    Triangle::translate( float x, float y, float z )
{
    m_pclMesh->translateRelative( VectorType3( x, y, z ) );
};