//=============================================================================
//
//  CLASS Triangle
//
//=============================================================================

#ifndef RAYTRACER_TRIANGLE_H
#define RAYTRACER_TRIANGLE_H

//== INCLUDES =================================================================
#include "Surface.h"
#include "TriangleSoup.h"

//== FORWARD REFS ================================================================

class TriangleSoup;


//== CLASS DEFINITION =========================================================

/*!
	A Triangle

*/
class Triangle: public Surface
{
//-------------------------------------------------------------- private data
private:	
	int m_i3VertexIndex[3];
	int m_i3NormalIndex[3];

	TriangleSoup* m_pclMesh;
//-------------------------------------------------------------- private methods
private:

//-------------------------------------------------------------- constructors
public:
	//! initializing constructor
	Triangle(  TriangleSoup* pclMesh,
		       int   i3VertexIndex[3],
			   int   i3NormalIndex[3]
			) ;


//-------------------------------------------------------------- element access

	AABBox getAABB() const;


//--------------------------------------------------------------- intersections

    bool intersect( const Ray&       clRay, 
	   			    const double     t0, 
					const double     t1, 
				    TracingContext&   tContext ) const;
				    
	
    //render object in OpenGL
    void    renderGL() const;

    //translate object
    void    translate( float, float, float );

    //rotate object
    void    rotate( float, float, float )
    {
        //not implemented yet
    };
    
    //Access to the Indices
    int getNormalIndex( unsigned int i ) const;
    int getVertexIndex( unsigned int i ) const;
    
    
    //Overwrite Selection Functions from Surface-class
    void setSelected( bool    sel );
    bool getSelected() const;

    //get Translation
    void getTranslation(
        float   *x,     
        float   *y,     
        float   *z )    
    {
        *x = 0;
        *y = 0;
        *z = 0;
    };

    //get Rotation
    void getRotation(
        int *rotx,  
        int *roty,  
        int *rotz ) 
    {
        *rotx = 0;
        *roty = 0;
        *rotz = 0;
    };

};

//=============================================================================
#endif // RAYTRACER_TRIANGLE_H
//=============================================================================

